<?php
namespace Ak\Fs\Image;

use Ak\Image\Image;
use Ak\Image\Processor\ProcessorInterface;
use Ak\Image\Processor\ResizeMode;
use League\Flysystem\Adapter\Local;
use League\Flysystem\Config;

/**
 * Omogućava pristup i obradu slika.
 *
 * @package Ak\Fs\Image
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/fs-image
 */
final class ImageAdapter extends Local
{
    /** @var ProcessorInterface */
    private $processor;

    /**
     * Konstruktor.
     *
     * @param string $root
     * @param ProcessorInterface $processor
     */
    public function __construct(
        string $root,
        ProcessorInterface $processor = null
    ) {
        parent::__construct($root);
        $this->processor = $processor;
    }

    /**
     * Kada gledamo da li slika postoji, zanima nas samo sirova slika
     * jer ostale možemo izgenerirati po potrebi.
     *
     * @inheritdoc
     */
    public function has($path)
    {
        $path = new ImagePath($path);
        return parent::has($path->getRawPath());
    }

    /**
     * @inheritdoc
     */
    public function getMimetype($path)
    {
        $path = new ImagePath($path);
        return parent::getMimetype($path->getRawPath());
    }

    /**
     * @inheritdoc
     */
    public function read($path)
    {
        $path = new ImagePath($path);

        if ($path->getResizeMode()) {
            return $this->readResized($path);
        }

        return $this->readOptimized($path);
    }

    /**
     * Vraća optimiziranu sliku originalnih dimenzija.
     *
     * @param ImagePath $path
     * @return array|bool|false
     */
    private function readOptimized(ImagePath $path)
    {
        $this->checkOptimized($path);
        return parent::read($path->getOptimizedPath());
    }

    /**
     * @param ImagePath $path
     * @return array|bool|false
     */
    private function readResized(ImagePath $path)
    {
        $this->checkResized($path);
        return parent::read($path->getResizedPath());
    }

    /**
     * @inheritdoc
     */
    public function readStream($path)
    {
        $path = new ImagePath($path);

        if (in_array($path->getResizeMode(), ResizeMode::getValues())) {
            return $this->readResizedStream($path);
        }

        return $this->readOptimizedStream($path);
    }

    /**
     * Vraća optimiziranu sliku originalnih dimenzija.
     *
     * @param ImagePath $path
     * @return array|bool|false
     */
    private function readOptimizedStream(ImagePath $path)
    {
        $this->checkOptimized($path);
        return parent::readStream($path->getOptimizedPath());
    }

    /**
     * @param ImagePath $path
     * @return array|false
     */
    private function readResizedStream(ImagePath $path)
    {
        $this->checkResized($path);
        return parent::readStream($path->getResizedPath());
    }

    /**
     * Provjerava da li postoji optimizirana slika originalnih dimenzija, ako ne
     * postoji, generira ju.
     *
     * @param ImagePath $path
     */
    private function checkOptimized(ImagePath $path)
    {
        $rawFilename = $this->applyPathPrefix($path->getRawPath());
        $filename = $this->applyPathPrefix($path->getOptimizedPath());

        if (!file_exists($filename)) {
            $raw = new Image(file_get_contents($rawFilename));
            $optimized = $this->processor->optimize($raw);

            parent::write($path->getOptimizedPath(), $optimized->getContents(), new Config());
        }
    }

    /**
     * Provjerava da li postoji slika datih dimenzija. Ako ne postoji generirat
     * će ju.
     *
     * @param ImagePath $path
     */
    private function checkResized(ImagePath $path)
    {
        $rawFilename = $this->applyPathPrefix($path->getRawPath());
        $filename = $this->applyPathPrefix($path->getResizedPath());

        if (!file_exists($filename)) {
            $raw = new Image(file_get_contents($rawFilename));
            $resized = $this->processor->resize($raw, $path->getWidth(), $path->getHeight(), $path->getResizeMode());

            parent::write($path->getResizedPath(), $resized->getContents(), new Config());
        }
    }

    /**
     * Samo sirove slike zapisujemo izvana. Sve ostale zapisujemo
     * iz ovog adaptera.
     *
     * @inheritdoc
     */
    public function write($path, $contents, Config $config)
    {
        $path = new ImagePath($path);
        return parent::write($path->getRawPath(), $contents, $config);
    }

    /**
     * @inheritdoc
     */
    public function writeStream($path, $resource, Config $config)
    {
        $path = new ImagePath($path);
        return parent::writeStream($path->getRawPath(), $resource, $config);
    }

    /**
     * @inheritdoc
     */
    public function update($path, $contents, Config $config)
    {
        $this->deleteGenerated($path);

        $path = new ImagePath($path);
        return parent::update($path->getRawPath(), $contents, $config);
    }

    /**
     * @inheritdoc
     */
    public function updateStream($path, $resource, Config $config)
    {
        $this->deleteGenerated($path);

        $path = new ImagePath($path);
        return parent::updateStream($path->getRawPath(), $resource, $config);
    }

    /**
     * @inheritdoc
     */
    public function rename($path, $newpath)
    {
        $this->deleteGenerated($path);

        $path = new ImagePath($path);
        $newpath = new ImagePath($newpath);
        return parent::rename($path->getRawPath(), $newpath->getRawPath());
    }

    /**
     * @inheritdoc
     */
    public function copy($path, $newpath)
    {
        $path = new ImagePath($path);
        $newpath = new ImagePath($newpath);
        return parent::copy($path->getRawPath(), $newpath->getRawPath());
    }

    /**
     * @inheritdoc
     */
    public function delete($path)
    {
        $this->deleteGenerated($path);

        $path = new ImagePath($path);
        $success = parent::delete($path->getRawPath());

        $dirname = basename(dirname($path->getRawPath()));

        if (count($this->listContents($dirname)) === 0) {
            parent::deleteDir($dirname);
        }

        return $success;
    }

    /**
     * Briše sve generirane datoteke.
     *
     * @param string $path
     */
    private function deleteGenerated(string $path): void
    {
        $path = new ImagePath($path);

        foreach (ResizeMode::getValues() as $mode) {
            foreach ($this->listContents($mode) as $size) {
                $filename = sprintf('%s/%s', $size['path'], $path->getPath());

                if (parent::has($filename)) {
                    parent::delete($filename);
                }

                if (count($this->listContents($size['path'])) === 0) {
                    parent::deleteDir($size['path']);
                }
            }

            if (count($this->listContents($mode)) === 0) {
                parent::deleteDir($mode);
            }
        }

        parent::delete($path->getOptimizedPath());
        $dirname = basename(dirname($path->getOptimizedPath()));

        if (count($this->listContents($dirname)) === 0) {
            parent::deleteDir($dirname);
        }
    }
}