<?php
namespace AkTest\Fs\Image;

use Ak\Fs\Adapter\AdapterPluginManager;
use Ak\Fs\Image\ImageAdapter;
use Ak\Image\Image;
use Ak\Image\Processor\ProcessorInterface;
use Ak\Test\AbstractTestCase;
use Laminas\ServiceManager\AbstractPluginManager;
use stdClass;

/**
 * Class ImageAdapterTest
 *
 * @package AkTest\Fs\Image
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/fs-image
 */
final class ImageAdapterTest extends AbstractTestCase
{
    public $providers = [
        TestConfigProvider::class,
    ];

    protected function setUp()
    {
        parent::setUp();

        $adapter = $this->getAdapter();
        $adapter->copy('flowers.jpg', 'image.jpg');
    }

    protected function tearDown()
    {
        parent::tearDown();

        $adapter = $this->getAdapter();

        if ($adapter->has('image.jpg')) {
            $adapter->delete('image.jpg');
        }
    }

    /**
     * @param null $processor
     * @return ImageAdapter
     */
    private function getAdapter($processor = null): ImageAdapter
    {
        $container = $this->getContainer();
        /** @var AbstractPluginManager $adapters */
        $adapters = $container->get(AdapterPluginManager::class);
        return $adapters->get('image', array_filter([
            'root' => __DIR__ . '/data',
            'processor' => $processor,
        ]));
    }

    public function testIsRegistered()
    {
        $container = $this->getContainer();
        /** @var AdapterPluginManager $adapters */
        $adapters = $container->get(AdapterPluginManager::class);

        $this->assertTrue($adapters->has('image'));
        /** @var ImageAdapter $adapter */
        $adapter = $adapters->get('image', [
            'root' => __DIR__ . '/data',
        ]);

        $this->assertInstanceOf(ImageAdapter::class, $adapter);
        $this->assertTrue($adapter->has('flowers.jpg'));
    }

    /**
     * @expectedException \Laminas\ServiceManager\Exception\ServiceNotCreatedException
     */
    public function testWillThrowExceptionForInvalidProcessorName()
    {
        $this->getAdapter('invalid_processor');
    }

    /**
     * @expectedException \Laminas\ServiceManager\Exception\ServiceNotCreatedException
     */
    public function testWillThrowExceptionForInvalidProcessor()
    {
        $this->getAdapter(new stdClass());
    }

    public function testCanGetMimeType()
    {
        $adapter = $this->getAdapter();
        $this->assertEquals('image/jpeg', $adapter->getMimetype('flowers.jpg')['mimetype']);
    }

    public function testWillReadOptimizedImageIfExists()
    {
        $adapter = $this->getAdapter();
        $file = $adapter->read('flowers.jpg');

        $this->assertEquals('presaved optimized image data', trim($file['contents']));
    }

    public function _testWillReadResizedImageIfExists()
    {
        $adapter = $this->getAdapter();
        $file = $adapter->read('thumb/100x100/flowers.jpg');

        $this->assertEquals('presaved resized image data', trim($file['contents']));
    }

    public function testWillCreateOptimizedImageIfDoesntExist()
    {
        $processor = $this->createMock(ProcessorInterface::class);
        $processor->expects($this->once())
            ->method('optimize')
            ->willReturn(new Image('optimized-data'));

        $adapter = $this->getAdapter($processor);
        $file = $adapter->read('image.jpg');

        $this->assertEquals('optimized-data', $file['contents']);
        $this->assertFileExists(__DIR__ . '/data/optimized/image.jpg');
    }

    public function testWillCreateResizedImageIfDoesntExist()
    {
        $processor = $this->createMock(ProcessorInterface::class);
        $processor->expects($this->once())
            ->method('resize')
            ->willReturn(new Image('resized-data'));

        $adapter = $this->getAdapter($processor);
        $file = $adapter->read('thumb/100x100/image.jpg');

        $this->assertEquals('resized-data', $file['contents']);
        $this->assertFileExists(__DIR__ . '/data/thumb/100x100/image.jpg');
    }
}
